# Adjust noise and voronoi settings for blender4!

bl_info = {
    "name": "Fracture Iterator",
    "description": "ça casse tout",
    "author": "Pictofilmo",
    "version": (1,7,0),
    "blender": (4, 2, 0),
    "location": "3D View > FRACTURE",
    "warning": "", # used for warning icon and text in addons panel
    "wiki_url": "",
    "tracker_url": "",
    "category": "Dynamics"
}


import bpy
import bmesh
from bpy.props import (StringProperty,
                       BoolProperty,
                       IntProperty,
                       FloatProperty,
                       FloatVectorProperty,
                       EnumProperty,
                       PointerProperty,
                       )
from bpy.types import (Panel,
                       Menu,
                       Operator,
                       AddonPreferences,
                       PropertyGroup,
                       )
from . import translate

# ------------------------------------------------------------------------
#    Scene Properties
# ------------------------------------------------------------------------
def get_material_items(self, context):
    # Generate EnumProperty items based on current materials in the scene
    return [(mat.name, mat.name, "") for mat in bpy.data.materials]  

    
class MyPropertiez(PropertyGroup):
    my_enum:EnumProperty(
        items=get_material_items,
        name="",
        description="Choose a material from the scene.",
    )
    
    
    my_scalx: FloatProperty(
        name = "",
        description = "Voronoï cell X scale",
        default = 1.0,
        min = 0,
        max = 1
        )       
    my_scaly: FloatProperty(
        name = "",
        description = "Voronoï cell Y scale",
        default = 1.0,
        min = 0,
        max = 1
        )  
    my_scalz: FloatProperty(
        name = "",
        description = "Voronoï cell Z scale. Set it to 0.01 to obtain wood shards",
        default = 1.0,
        min = 0,
        max = 1
        )                  
    my_enu: EnumProperty(
        name="",
        description="Fracturation modes",
        items=[ ('PARTICLE_OWN', "Particles", "Volume scattered based fragmentation"),
                ('PENCIL', "Pencil", "Annotation Pencil based fragmentation"),
                ('P3', "Pencil + Particles", "Mix of Volume scattered and Pencil based fragmentation"),
                ('VERT_OWN', "Vertex", "Vertex based fragmentation"),
                ('VERT_CHILD', "Child Vertex", "Child objects vertex based fragmentation "),
                ('P0', "Manual   Settings", ""),
               ]
        )
    my_num: IntProperty(
        name = "Count ",
        description = "Approx number of cells after fracturation",
        default = 20,
        min = 0,
        max = 10000
        )   
        
      
    my_start: IntProperty(
        name = "Start",
        description = "First frame for the custom timeline",
        default = 0,
        min = 0,
        max = 10000000
        )
        
    my_end: IntProperty(
        name = "End",
        description = "Last frame for the custom timeline",
        default = 250,
        min = 0,
        max = 10000000
        )
                    
    my_friction: FloatProperty(
        name = "Rigid Bodies Friction",
        description = "Adjust the friction value for produced Rigid Bodies",
        default = 0.8,
        min = 0,
        max = 1
        )
        
    my_boolCollection: BoolProperty(
        name="Collection",
        description="If checked, every elements will go to the named collection",
        default = True
        )
    my_boolTimeline: BoolProperty(
        name="Use Custom Timeline",
        description="Bake and calculation will be done in the scene timeline limits",
        default = False
        )

    my_name: StringProperty(
        name="Collection",
        description="Collection where all elements will end",
        default="FractureIterator",
        maxlen=1024,
        )
    my_boolAutoplay: BoolProperty(
        name="AutoPlay",
        description="Play animation after computation",
        default = True
        )
    my_boolTexspc: BoolProperty(
        name="Unified Texture Space",
        description="Unify texture space at Break operation",
        default = True
        )              
    my_boolMat: BoolProperty(
        name="Material",
        description="Enable to choose a specific material for the cuts. If not checked, a Break_mat will be generated",
        default = True
        ) 
  
    my_streng: FloatProperty(
        name = "Strength",
        description = "",
        default = 0.25,
        min = 0.001,
        max = 8
        ) 
    my_scal: FloatProperty(
        name = "Scale",
        description = "",
        default = 0.18,
        min = 0.001,
        max = 8
        )    
    my_subd: IntProperty(
        name = "Viewport",
        description = "",
        default = 2,
        min = 0,
        max = 5
        )      
    my_subd2: IntProperty(
        name = "Render",
        description = "",
        default = 2,
        min = 0,
        max = 5
        )    








# ------------------------------------------------------------------------
#    FONCTIONS
# ------------------------------------------------------------------------

def value_driver(ob,feed):
    #ex: value_driver(bck1.inputs[1],'prop.col2power')
    drc = ob.driver_add("default_value")
    drc.driver.expression = 'var'
    var = drc.driver.variables.new()
    var.name='var'
    var.type='SINGLE_PROP'
    var.targets[0].id_type = 'SCENE'
    var.targets[0].id = bpy.context.scene
    var.targets[0].data_path=feed  
    return


def break_group():
    ng = bpy.data.node_groups
    if 'break_group'in ng:
        pass
        #print("break_group already there")
    else:
        break_group = bpy.data.node_groups.new('break_group', 'ShaderNodeTree')
        break_group.nodes.clear()
        
        group_outputs = break_group.nodes.new('NodeGroupOutput')
    
        out_col=break_group.interface.new_socket(name="Color", in_out='OUTPUT',socket_type='NodeSocketColor')
        #out_col=break_group.outputs.new('NodeSocketColor','Color')  
        out_norm=break_group.interface.new_socket(name="Normal", in_out='OUTPUT',socket_type='NodeSocketVector')
        #out_norm=break_group.outputs.new('NodeSocketVector','Normal')  
        group_outputs.location = (1000,0)
        group_inputs = break_group.nodes.new('NodeGroupInput')
        
        in_scal=break_group.interface.new_socket(name="Scale", in_out='INPUT',socket_type='NodeSocketFloat')
        #in_scal=break_group.inputs.new('NodeSocketFloat','Scale')
        in_scal.default_value = .49
        in_scal.min_value = 0.001
        in_scal.max_value = 10
        
        in_bmp=break_group.interface.new_socket(name="Bump", in_out='INPUT',socket_type='NodeSocketFloat')
        #in_bmp=break_group.inputs.new('NodeSocketFloat','Bump')
        in_bmp.default_value = .6
    
        in_col1=break_group.interface.new_socket(name="Color1", in_out='INPUT',socket_type='NodeSocketColor')
        #in_col1=break_group.inputs.new('NodeSocketColor','Color1')
        in_col1.default_value = .7,.7,.7,1
        
        in_col2=break_group.interface.new_socket(name="Color1", in_out='INPUT',socket_type='NodeSocketColor')
        #in_col2=break_group.inputs.new('NodeSocketColor','Color1')
        in_col2.default_value = .12,.12,.12,1        

        group_inputs.location = (-1000,0)
        
        n01= break_group.nodes.new('ShaderNodeMix')
        n01.location = (-1000,-200)
        n01.inputs[0].default_value = .5
        n01.inputs[2].default_value = 0.001
        n01.inputs[3].default_value = 10
        n01.label="n01"
        
        n02= break_group.nodes.new('ShaderNodeMath')
        n02.location = (-800,-200)
        n02.label="n02"
        n02.inputs[1].default_value = 2
        n02.operation = 'MULTIPLY'

        n03= break_group.nodes.new('ShaderNodeTexMusgrave')
        n03.location = (-600,100)
        n03.label="n03"
        n03.musgrave_type = 'HETERO_TERRAIN'
        n03.inputs[3].default_value = 6
        n03.inputs[4].default_value = 0.22
        n03.inputs[5].default_value = 1.89

        n04= break_group.nodes.new('ShaderNodeTexMusgrave')
        n04.location = (-600,-250)
        n04.label="n04"
        n04.musgrave_type = 'HETERO_TERRAIN'
        n04.inputs[3].default_value = 3.5
        n04.inputs[4].default_value = 1.5
        n04.inputs[5].default_value = 2.2

        n05= break_group.nodes.new('ShaderNodeMath')
        n05.location = (-400,-50)
        n05.label="n05"
        n05.operation = 'MINIMUM'

        n06= break_group.nodes.new('ShaderNodeBump')
        n06.location = (-200,-50)
        n06.label="n06"
        n06.inputs[0].default_value = 0.62

        n07= break_group.nodes.new('ShaderNodeMath')
        n07.location = (-600,400)
        n07.label="n07"
        n07.inputs[1].default_value = 2
        n07.operation = 'MULTIPLY'
        n07.inputs[1].default_value = 20

        n08= break_group.nodes.new('ShaderNodeTexNoise')
        n08.location = (-400,400)
        n08.label="n08"
        n08.inputs[3].default_value = 5
        n08.inputs[4].default_value = 0.5
        n08.inputs[5].default_value = 2




        n07b= break_group.nodes.new('ShaderNodeMath')
        n07b.location = (-600,100)
        n07b.label="n07b"
        n07b.inputs[1].default_value = 2
        n07b.operation = 'MULTIPLY'
        n07b.inputs[1].default_value = 2
        
        n08b= break_group.nodes.new('ShaderNodeTexNoise')
        n08b.location = (-200,600)
        n08b.label="n08b"
        n08b.inputs[3].default_value = 5
        n08b.inputs[4].default_value = 0.5
        n08b.inputs[5].default_value = 2

        n09b= break_group.nodes.new('ShaderNodeMix')
        n09b.location = (-100,100)
        n09b.data_type = 'RGBA'
        n09b.blend_type = 'MULTIPLY'
        n09b.inputs[0].default_value = 1
        n09b.label="n09b"






        

        n09= break_group.nodes.new('ShaderNodeMix')
        n09.location = (-100,400)
        n09.data_type = 'RGBA'
        n09.inputs[6].default_value = (0.440552, 0.440552, 0.440552, 1)
        n09.inputs[7].default_value = (0.181152, 0.181152, 0.181152, 1)
        n09.label="n09"
        
        
        break_group.links.new(n01.outputs[0], n02.inputs[0])        
        break_group.links.new(n01.outputs[0], n03.inputs[2])  
        break_group.links.new(n02.outputs[0], n04.inputs[2])  
        break_group.links.new(n04.outputs[0], n05.inputs[1]) 
        break_group.links.new(n03.outputs[0], n05.inputs[0]) 
        break_group.links.new(n05.outputs[0], n06.inputs[2]) 
        break_group.links.new(n06.outputs[0], group_outputs.inputs[1]) 
        break_group.links.new(n01.outputs[0], n07.inputs[0]) 
        break_group.links.new(n07.outputs[0], n08.inputs[2])       
        break_group.links.new(n08.outputs[0], n09b.inputs[6]) 
        break_group.links.new(n08b.outputs[0], n09b.inputs[7]) 
        break_group.links.new(n09b.outputs[2], n09.inputs[0]) 
        break_group.links.new(n07.outputs[0], n07b.inputs[0]) 
        break_group.links.new(n07b.outputs[0], n08b.inputs[2]) 
    
        break_group.links.new(n09.outputs[2], group_outputs.inputs[0])   
        break_group.links.new(group_inputs.outputs[0], n01.inputs[0])   
        break_group.links.new(group_inputs.outputs[1], n06.inputs[0] )
        break_group.links.new(group_inputs.outputs[2], n09.inputs[6] )
        break_group.links.new(group_inputs.outputs[3], n09.inputs[7] )

# ------------------------------------------------------------------------
#    Buttons operators
# ------------------------------------------------------------------------


# 2 times old settings ##################################################################
class FRACTUREButton(Operator):
    bl_idname = "wm.fracture"
    bl_label = "1:: Fracture"
    bl_description="Break your selected object with Cell Fracture addon. Cell Fracture addon (by ideasman42,phymec,Sergey Sharybin) has to be activated in Blender Addons"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):
        scene = bpy.context.scene
        mytool = scene.toolz  
        count= mytool.my_num
        scalx= mytool.my_scalx
        scaly= mytool.my_scaly
        scalz= mytool.my_scalz  
        

        
        bpy.ops.object.add_fracture_cell_objects('INVOKE_DEFAULT',
        use_debug_redraw=False, 
        collection_name=mytool.my_name,
        material_index=1,
        use_sharp_edges_apply=False,
        cell_scale=(scalx, scaly, scalz),            
        source_limit=count
        )
                
        
        return {'FINISHED'}  


class BREAK0Button(Operator):
    bl_idname = "wm.breaker0"
    bl_label = "2:: B R E A K !"
    bl_description="Give Physics to the fragments from the actual frame. Use it right after using Fracture Panel"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):
        scene = bpy.context.scene
        mytool = scene.toolz  
        
        

            
                
                
                
        if mytool.my_boolTimeline == False:
            start_frame=scene.frame_start
            end_frame=scene.frame_end
        else:
            start_frame=mytool.my_start
            end_frame=mytool.my_end       
        break_frame=scene.frame_current        
        friction= mytool.my_friction
        
        ### create collection and make it active ################################
        if mytool.my_boolCollection == True:
            colname = mytool.my_name
            colname2 = colname+"_hide"
            mastercollection = bpy.context.scene.collection  
            res=0
            res2=0
            for collec in bpy.data.collections:
                if collec.name == colname:
                    col = bpy.data.collections.get(colname)
                    res+=1
                if collec.name == colname2:
                    col = bpy.data.collections.get(colname2)
                    res2+=1                
            if res<1:    
                mycollection = bpy.data.collections.new(name=colname)
                bpy.context.scene.collection.children.link(mycollection) 
            if res2<1:    
                mycollection = bpy.data.collections.new(name=colname2)
                bpy.context.scene.collection.children.link(mycollection)                             
            layer_collection = bpy.context.view_layer.layer_collection
            for layer in layer_collection.children:
                if layer.name == colname:
                    bpy.context.view_layer.active_layer_collection = layer
            bpy.data.collections[colname2].hide_render = True
            bpy.data.collections[colname2].hide_viewport = True


    
        src = bpy.context.active_object
        src.select_set(False)
        fraglist=[]
        for i in bpy.context.selected_objects:
            fraglist.append(i)
            
        
        bpy.ops.object.select_all(action='DESELECT')
        src.select_set(True)
        
        # cleanup             
        try:    
            bpy.ops.gpencil.layer_annotation_remove() #delete grease pencil   
        except:
            pass
            #print("No pencil to erase")     
                  
        #bake visual transform
        #scene.frame_current = start_frame 
        #bpy.context.view_layer.update()
        bpy.ops.nla.bake(frame_start=start_frame, frame_end=end_frame, visual_keying=True, clear_parents=True, bake_types={'OBJECT'})
        bpy.ops.rigidbody.object_add() # remove RB (add one before if not any to avoid error)
        bpy.ops.rigidbody.object_remove()

        #empty creation
        objects = bpy.data.objects
        EMPT = objects.get('EMPT'+src.name)
        if EMPT is None:
            bpy.ops.object.empty_add(type='PLAIN_AXES')
            EMPT= bpy.context.active_object
            EMPT.name = 'EMPT_'+src.name
        EMPT.show_in_front=True  
        
        # copy ob action to EMPT
        bpy.ops.object.select_all(action='DESELECT')
        src.select_set(True)
        bpy.context.view_layer.objects.active = src
        context = bpy.context
        areatype = context.area.type
        bpy.context.area.type = 'DOPESHEET_EDITOR'
        bpy.context.space_data.mode = 'ACTION'
        EMPT.animation_data_create() # !!!!!!!!!!!!!!!!! coooool
        EMPT.animation_data.action = src.animation_data.action
        bpy.ops.action.unlink()
        bpy.context.area.type = areatype
        
        bpy.ops.object.select_all(action='DESELECT')
        # texture space
        active_ob = bpy.context.view_layer.objects.active
        for i in fraglist:
            i.data.use_auto_smooth = True

            if mytool.my_boolTexspc == True:
                i.data.use_auto_texspace = False
                i.data.texspace_location[0] = -i.matrix_world.translation[0]+active_ob.location[0]
                i.data.texspace_location[1] = -i.matrix_world.translation[1]+active_ob.location[1]
                i.data.texspace_location[2] = -i.matrix_world.translation[2]+active_ob.location[2]
                i.data.texspace_size[0] = 1
                i.data.texspace_size[1] = 1
                i.data.texspace_size[2] = 1
                
                
        # rigid bodies
        for obj in fraglist:
            ##print(obj.name)
            obj.select_set(True)
            bpy.context.view_layer.objects.active = obj
            bpy.ops.rigidbody.objects_add(type='ACTIVE') 
            bpy.context.object.rigid_body.kinematic = True
            bpy.context.object.rigid_body.friction = friction
          

        #parent ob to EMPT
        EMPT.select_set(True)
        bpy.context.view_layer.objects.active = EMPT
        bpy.ops.object.parent_set()

        #add kinematic key on/off
        EMPT.select_set(False)
        end = break_frame
        for i in fraglist:
            bpy.context.view_layer.objects.active=i
            scene.frame_current = end-1
            i.rigid_body.kinematic = True
            i.keyframe_insert('rigid_body.kinematic')
            scene.frame_current = end
            i.rigid_body.kinematic = False
            i.keyframe_insert('rigid_body.kinematic') 
         
        #collection 
        if mytool.my_boolCollection == True:
            col = bpy.data.collections.get(colname)
            EMPT.select_set(True )
            sel = bpy.context.selected_objects
            
            for o in sel :
                origin_collname= o.users_collection[0].name 
                #print(o.name, origin_collname)
                origin_coll = bpy.data.collections.get(origin_collname)
                if origin_coll != None:
                    origin_coll.objects.unlink(o)
                    col.objects.link(o)   
                else:
                    mastercollection.objects.unlink(o)
                    col.objects.link(o)  
                     
            col2 = bpy.data.collections.get(colname2)
            origin_collname= src.users_collection[0].name 
            origin_coll = bpy.data.collections.get(origin_collname)
#            if origin_coll != None:
#                origin_coll.objects.unlink(src)
#                col2.objects.link(src)   
#            else:
#                mastercollection.objects.unlink(src)
#                col.objects.link(src) 

        EMPT.hide_viewport = True
        EMPT.select_set(False)
        if mytool.my_boolAutoplay == True:
            scene.frame_current = start_frame
            bpy.ops.screen.animation_play()

        return {'FINISHED'} 
    

# one click preset settings ##################################################################

class BREAKButton(Operator):
    bl_idname = "wm.breaker"
    bl_label = "VORONOI BREAK !"
    bl_description="Voronoï fracturation based on above settings then trigger physics at current frame"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):
        scene = bpy.context.scene
        mytool = scene.toolz  
        count= mytool.my_num
        scalx= mytool.my_scalx
        scaly= mytool.my_scaly
        scalz= mytool.my_scalz
        bpy.context.space_data.overlay.show_relationship_lines = False
        
        index=0
        try:
            bpy.ops.preferences.addon_enable(module="bl_ext.blender_org.cell_fracture")
            index+=1
            bpy.ops.extensions.repo_refresh_all()
        except:
            print("Nop")
        try:
            bpy.ops.preferences.addon_enable(module="bl_ext.user_default.cell_fracture")
            index+=1
            bpy.ops.extensions.repo_refresh_all()
        except:
            print("Nop nop")            
        if index >0:
            #src = bpy.context.active_object
            
            if mytool.my_boolMat == False:        
                #create materials
                mat_name="Break_mat"
                ng = bpy.data.materials
                if mat_name in ng:
                    material = bpy.data.materials.get(mat_name)
                else:            
                    break_group()
                    material = bpy.data.materials.new(name=mat_name)
                    material.diffuse_color = (0.25, 0.25, 0.25,1) 
                    material.use_nodes = True
                    nodes = material.node_tree.nodes
                    material.node_tree.nodes.clear() 
                    princ = nodes.new('ShaderNodeBsdfPrincipled')
                    princ.location = (0,0)
                    princ.inputs[7].default_value = 0
                    outepute= nodes.new('ShaderNodeOutputMaterial')
                    outepute.location=(300,0)
                    grp = nodes.new('ShaderNodeGroup',)
                    grp.node_tree = bpy.data.node_groups['break_group']
                    grp.location = (-400,0)
                    grp.inputs[0].default_value =.49
                    grp.inputs[1].default_value =.3
                    grp.inputs[2].default_value = (0.0601579, 0.0601579, 0.0601579, 1)
                    grp.inputs[3].default_value = (0.668941, 0.668941, 0.668941, 1)
                    
                    princ.inputs[12].default_value = 1
                    princ.inputs[3].default_value = 1

                    material.node_tree.links.new(grp.outputs[0], princ.inputs[0])
                    material.node_tree.links.new(grp.outputs[1], princ.inputs[5])
                    material.node_tree.links.new(princ.outputs[0], outepute.inputs[0])                          
            else:
                mat_name=mytool.my_enum                
                material = bpy.data.materials.get(mat_name)
                



            for o in bpy.context.selected_objects:
                sluts=len(o.material_slots)
                if sluts<2:
                    o.data.materials.append(material) 
                else:
                    if mytool.my_boolMat == True:                
                        mat_name=mytool.my_enum
                        new_material = bpy.data.materials.get(mat_name)
                        o.data.materials[1] = new_material
                        bpy.ops.wm.redraw_timer(type='DRAW_WIN_SWAP', iterations=1)
                    else:
                        mat_name="Break_mat"
                        new_material = bpy.data.materials.get(mat_name)
                        o.data.materials[1] = new_material
                        bpy.ops.wm.redraw_timer(type='DRAW_WIN_SWAP', iterations=1)                    
                        
    #                replace index1 by material
                    
            
            sel=bpy.context.selected_objects
            break_frame=scene.frame_current 
            MEGAfraglist=[]
            for obj in sel:
                bpy.context.view_layer.objects.active = obj
                src= obj
                        # force single object operation
                bpy.ops.object.select_all(action='DESELECT')
                src.select_set(True)
                bpy.context.view_layer.objects.active = src 
                
                # action bake setting              
                if mytool.my_boolTimeline == False:
                    start_frame=scene.frame_start
                    end_frame=scene.frame_end
                else:
                    start_frame=mytool.my_start
                    end_frame=mytool.my_end       
                #break_frame=scene.frame_current        
                friction= mytool.my_friction
                
                ### create collection and make it active ################################
                if mytool.my_boolCollection == True:
                    colname = mytool.my_name
                    colname2 = colname+"_hide"
                    mastercollection = bpy.context.scene.collection  
                    active_befor = bpy.context.view_layer.active_layer_collection
                    res=0
                    res2=0
                    for collec in bpy.data.collections:
                        if collec.name == colname:
                            col = bpy.data.collections.get(colname)
                            res+=1
                        if collec.name == colname2:
                            col = bpy.data.collections.get(colname2)
                            res2+=1                
                    if res<1:    
                        mycollection = bpy.data.collections.new(name=colname)
                        bpy.context.scene.collection.children.link(mycollection) 
                    if res2<1:    
                        mycollection = bpy.data.collections.new(name=colname2)
                        bpy.context.scene.collection.children.link(mycollection)                             
                    layer_collection = bpy.context.view_layer.layer_collection
                    for layer in layer_collection.children:
                        if layer.name == colname:
                            bpy.context.view_layer.active_layer_collection = layer
                    bpy.data.collections[colname2].hide_render = True
                    bpy.data.collections[colname2].hide_viewport = True

                # Bake src action
                bpy.ops.nla.bake(frame_start=start_frame, frame_end=end_frame, visual_keying=True, clear_parents=True, bake_types={'OBJECT'})
                #Rough cuts eraser
                obj.modifiers.clear()
                #bpy.ops.object.delete_all_modifiers()
                try:
                    src.rigid_body.type!=None
                    bpy.ops.rigidbody.object_remove()
                except:
                    pass
                    #print("No RB to remove") 
                    

                #particles
                if mytool.my_enu == 'PARTICLE_OWN' or mytool.my_enu == 'P3':
                    bframe = bpy.context.scene.frame_current                                   
                    ps = src.modifiers.new("pouetpouet", 'PARTICLE_SYSTEM')
                    psys = src.particle_systems[ps.name]
                    psys.settings.count = count
                    psys.settings.emit_from = 'VOLUME'
                    psys.settings.frame_start = bframe
                    psys.settings.frame_end = bframe
                    psys.settings.display_method = 'CROSS'
                    #psys.seed = random.randrange(100)
                if mytool.my_enu == 'P3':      
                    #cell fracture
                    bpy.ops.object.add_fracture_cell_objects(
                    use_debug_redraw=False, 
                    #collection_name=src .name,
                    source_limit=count,
                    material_index=1,
                    margin=0.000,
                    use_sharp_edges_apply=False,
                    cell_scale=(scalx, scaly, scalz),            
                    source={'PENCIL','PARTICLE_OWN'})  
                else:
                    #cell fracture
                    bpy.ops.object.add_fracture_cell_objects(
                    use_debug_redraw=False, 
                    #collection_name=src .name,
                    source_limit=count,
                    material_index=1,
                    margin=0.000,
                    use_smooth_faces=True,
                    use_sharp_edges_apply=False,
                    cell_scale=(scalx, scaly, scalz),            
                    source={mytool.my_enu})  
                
                #listing frags
                src.select_set(False)
                fraglist=[]
                for i in bpy.context.selected_objects:
                    bpy.context.view_layer.objects.active = i
                    fraglist.append(i)
                
                # cleanup        
                src.select_set(True)
                bpy.context.view_layer.objects.active = src    
                if mytool.my_enu == 'PARTICLE_OWN' or mytool.my_enu == 'P3':  
                    #bpy.ops.object.delete_all_modifiers() #delete particle system   
                    try:    
                        bpy.ops.object.modifier_remove(modifier = "pouetpouet")    
                    except:
                        pass
                        #print("No particles here")     
                        
                if mytool.my_enu == 'PENCIL'or mytool.my_enu == 'P3': 
                    try:    
                        bpy.ops.gpencil.layer_annotation_remove() #delete grease pencil   
                    except:
                        pass
                        #print("No pencil to erase")     
                
                #empty creation
                objects = bpy.data.objects
                EMPT = objects.get('EMPT'+src.name)
                if EMPT is None:
                    bpy.ops.object.empty_add(type='PLAIN_AXES')
                    EMPT= bpy.context.active_object
                    EMPT.name = 'EMPT_'+src.name
                EMPT.show_in_front=True  
                
                # copy src action to EMPT
                bpy.ops.object.select_all(action='DESELECT')
                src.select_set(True)
                bpy.context.view_layer.objects.active = src

                #new_action = src.animation_data.action.copy()
                EMPT.animation_data_create()
                EMPT.animation_data.action = src.animation_data.action
                src.animation_data.action = None            
    #            context = bpy.context
    #            areatype = context.area.type
    #            bpy.context.area.type = 'DOPESHEET_EDITOR'
    #            bpy.context.space_data.mode = 'ACTION'
    #            EMPT.animation_data_create() # !!!!!!!!!!!!!!!!! coooool
    #            EMPT.animation_data.action = src.animation_data.action
    #            bpy.ops.action.unlink()            
    #            bpy.context.area.type = areatype
                
                
                bpy.ops.object.select_all(action='DESELECT')
                
                # texture space
                active_ob = bpy.context.view_layer.objects.active
                for i in fraglist:
                    #i.data.use_auto_smooth = True
                    if mytool.my_boolTexspc == True:
                        i.data.use_auto_texspace = False
                        i.data.texspace_location[0] = -i.matrix_world.translation[0]+active_ob.location[0]
                        i.data.texspace_location[1] = -i.matrix_world.translation[1]+active_ob.location[1]
                        i.data.texspace_location[2] = -i.matrix_world.translation[2]+active_ob.location[2]
                        i.data.texspace_size[0] = 1
                        i.data.texspace_size[1] = 1
                        i.data.texspace_size[2] = 1
                bpy.context.view_layer.objects.active=active_ob


                                        
                # add rigid bodies
                for obj in fraglist:
                    #print(obj.name)
                    obj.select_set(True)
                    bpy.context.view_layer.objects.active = obj
                    bpy.ops.rigidbody.objects_add(type='ACTIVE') 
                    bpy.context.object.rigid_body.kinematic = True
                    bpy.context.object.rigid_body.friction = friction
                    obj.rigid_body.mesh_source = 'BASE'
                    MEGAfraglist.append(obj)
                  
                #parent ob to EMPT
                EMPT.select_set(True)
                bpy.context.view_layer.objects.active = EMPT
                bpy.ops.object.parent_set()
                
                #add kinematic key on/off
                EMPT.select_set(False)
                end = break_frame
                for i in fraglist:
                    bpy.context.view_layer.objects.active=i
                    scene.frame_current = end-1
                    i.rigid_body.kinematic = True
                    i.keyframe_insert('rigid_body.kinematic')
                    scene.frame_current = end
                    i.rigid_body.kinematic = False
                    i.keyframe_insert('rigid_body.kinematic') 
                 

                col2 = bpy.data.collections.get(colname2)
                for c in bpy.data.collections:   
                    try:
                        c.objects.unlink(src)
                        col2.objects.link(src)
                        #print(c.name,"linked")
                    except:
                        pass
                        #print(c.name,"unlinked")        
                try:
                    bpy.context.scene.collection.objects.unlink(src)
                    #print("root unliked")
                except:
                    pass
                    #print("not in root")
              
                bpy.context.view_layer.active_layer_collection = active_befor
                
                for obj in MEGAfraglist:
                    obj.select_set(True)
                    bpy.ops.object.shade_auto_smooth()
                    wnorm = obj .modifiers.new(name="WeightedNormal",type='WEIGHTED_NORMAL')  
                    wnorm.keep_sharp = True
                    


                    
                EMPT.hide_viewport = True
                EMPT.select_set(False)
                    
                    
                if mytool.my_boolAutoplay == True:
                    scene.frame_current = start_frame
                    bpy.ops.screen.animation_play()
                
                
        else:
            print("cell fracture non disponible")
            def draw(self, context):
                self.layout.label(text="Please install Cell Fracture addon first")
            bpy.context.window_manager.popup_menu(draw, title = "MISSING ADDON", icon = 'INFO')    
            


        
        return {'FINISHED'} 


class roughButton(Operator):
    bl_idname = "wm.rough"
    bl_label = "Add to Selected"
    bl_description="Add rough cuts"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):
        mytool = bpy.context.scene.toolz
        # VOROCUTS   ###############################
        
        #bpy.ops.object.delete_all_modifiers()
        fraglist= bpy.context.selected_objects
#        if 'VoroCUT' in bpy.data.textures:
#            tex = bpy.data.textures.get("VoroCUT", 'CLOUDS')    
#        else:
#            tex = bpy.data.textures.new("VoroCUT", 'CLOUDS')
        
        if 'VoroCUT' in bpy.data.textures:            
            list=[]
            for o in bpy.data.textures:
                o_name=o.name
                if o_name.startswith("VoroCUT"):
                   list.append(o)
            l=len(list)
            nam="VoroCUT"+str(l+1)
            tex = bpy.data.textures.new(nam, 'CLOUDS')                      
            tex.noise_basis = 'VORONOI_F1'  
            tex.noise_scale = mytool.my_scal
        else:
            nam="VoroCUT"
            tex = bpy.data.textures.new(nam, 'CLOUDS')        
        
#        #DRIVER
#        drc = tex.driver_add('noise_scale')
#        drc.driver.expression = 'var'
#        var = drc.driver.variables.new()
#        var.name='var'
#        var.type='SINGLE_PROP'
#        var.targets[0].id_type = 'SCENE'
#        var.targets[0].id = bpy.context.scene
#        var.targets[0].data_path='toolz.my_scal'  
 
        
        
        
        
                              
        for ob in fraglist:
            
            bpy.ops.object.select_all(action='DESELECT')
            ob.select_set(True)
            bpy.context.view_layer.objects.active = ob
            ob.modifiers.clear()
            # UV unwrap
            bpy.ops.object.mode_set(mode="EDIT")    
            bpy.ops.mesh.select_mode(use_extend=False, use_expand=False, type='FACE')             
            bpy.ops.mesh.select_all(action='SELECT')
            bpy.ops.uv.smart_project()
            bpy.ops.mesh.faces_shade_smooth()
            bpy.ops.mesh.select_all(action="DESELECT")                
            for m in ob.material_slots:
                ob.active_material_index=0
                mat = bpy.context.object.active_material
                bpy.ops.object.material_slot_select()
                try:
                    bpy.ops.object.vertex_group_remove(all=True)
                except:
                    pass
                bpy.ops.object.vertex_group_assign_new()           
            bpy.ops.object.mode_set(mode="OBJECT")                                  

            #bpy.ops.object.delete_all_modifiers()
            submod = ob.modifiers.new(name="subd",type='SUBSURF')
            submod.subdivision_type = 'SIMPLE' 
            submod.levels = 0
            submod.render_levels = mytool.my_subd2
#            #DRIVER
#            drc = submod.driver_add('levels')
#            drc.driver.expression = 'var'
#            var = drc.driver.variables.new()
#            var.name='var'
#            var.type='SINGLE_PROP'
#            var.targets[0].id_type = 'SCENE'
#            var.targets[0].id = bpy.context.scene
#            var.targets[0].data_path='toolz.my_subd'
#            #DRIVER
#            drc = submod.driver_add('render_levels')
#            drc.driver.expression = 'var'
#            var = drc.driver.variables.new()
#            var.name='var'
#            var.type='SINGLE_PROP'
#            var.targets[0].id_type = 'SCENE'
#            var.targets[0].id = bpy.context.scene
#            var.targets[0].data_path='toolz.my_subd2'
            
                                
            dispmod = ob.modifiers.new(name="disp",type='DISPLACE')  
            dispmod.vertex_group = "Group"
            dispmod.strength=mytool.my_streng
#            #DRIVER
#            drc = dispmod.driver_add('strength')
#            drc.driver.expression = 'var'
#            var = drc.driver.variables.new()
#            var.name='var'
#            var.type='SINGLE_PROP'
#            var.targets[0].id_type = 'SCENE'
#            var.targets[0].id = bpy.context.scene
#            var.targets[0].data_path='toolz.my_streng'
        
            dispmod.invert_vertex_group = True
            dispmod.texture = bpy.data.textures[nam]
            
            
            decipmod = ob.modifiers.new(name="deci",type='DECIMATE') 
            decipmod.decimate_type = 'DISSOLVE'
            decipmod.angle_limit = 0.0872665

            triangmod = ob.modifiers.new(name="tria",type='TRIANGULATE') 
            triangmod.keep_custom_normals = True

             
        #########################################   
        for ob in fraglist:
            ob.select_set(True)
            bpy.context.view_layer.objects.active = ob  
              
        for ob in bpy.context.selected_objects:
            try:
                ob.modifiers["subd"].levels = mytool.my_subd
                #print(ob.name," subd")
            except:
                pass       
            
        return {'FINISHED'}  
    
    
class unroughButton(Operator):
    bl_idname = "wm.unrough"
    bl_label = "Remove"
    bl_description="Remove Rough Cuts for selected objects"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):
        sel=bpy.context.selected_objects
        for o in sel:
            for m in o.modifiers:
                if m.name=="disp":
                    o.modifiers.remove(o.modifiers.get("disp"))
                if m.name=="subd":
                    o.modifiers.remove(o.modifiers.get("subd"))       
                if m.name=="deci":
                    o.modifiers.remove(o.modifiers.get("deci"))      
                if m.name=="tria":
                    o.modifiers.remove(o.modifiers.get("tria"))      
        return {'FINISHED'}  
    
            
class resetroughButton(Operator):
    bl_idname = "wm.resetrough"
    bl_label = "Reset"
    bl_description="Reset Rough Cuts for all objects in the scene"
    bl_options = {'REGISTER', 'UNDO'}
 
    def execute(self, context):
        sel=bpy.data.objects
        for o in sel:
            for m in o.modifiers:
                if m.name=="subd":
                    m.driver_remove("levels")
                    m.driver_remove("render_levels")
                    o.modifiers.remove(o.modifiers.get("subd"))
                if m.name=="disp":
                    m.driver_remove("strength")   
                    o.modifiers.remove(o.modifiers.get("disp"))
                if m.name=="deci":
                    o.modifiers.remove(o.modifiers.get("deci")) 
                if m.name=="tria":
                    o.modifiers.remove(o.modifiers.get("tria"))   

#        message = " You didn't saved yet "
#        self.report({'INFO'}, message)
#        print(message)   

        return {'FINISHED'}  

    def invoke(self, context, event):

        return context.window_manager.invoke_props_dialog(self,width = 100)  

    
class updroughButton(Operator):
    bl_idname = "wm.updrough"
    bl_label = "Update"
    bl_description=""
    
    def execute(self, context):
        mytool = bpy.context.scene.toolz
        
        #bpy.data.textures["VoroCUT"].noise_scale = mytool.my_scal
        for ob in bpy.context.selected_objects:
            try:
                ob.modifiers["subd"].levels = 0
            except:
                pass 
            
        fraglist= bpy.context.selected_objects
        rough_mod=["subd","disp"] 
        fraglist= bpy.context.selected_objects
        
        for ob in fraglist:
            #print(ob.name)
            bpy.ops.object.select_all(action='DESELECT')
            ob.select_set(True)
            bpy.context.view_layer.objects.active = ob
            try:
                
                ob.modifiers["disp"].strength = mytool.my_streng
                nam=ob.modifiers["disp"].texture.name
                bpy.data.textures[nam].noise_scale = mytool.my_scal
                
                ob.modifiers["subd"].render_levels = mytool.my_subd2
            except:
                pass                                   
        for ob in fraglist:
            ob.select_set(True)
            bpy.context.view_layer.objects.active = ob
        for ob in bpy.context.selected_objects:
            try:
                ob.modifiers["subd"].levels = mytool.my_subd
                #print(ob.name," subd")
            except:
                pass             
        #print("over")
#        
#        for ob in fraglist:
#            for i in rough_mod:
#                for m in ob.modifiers:
#                    rough_mod = ob.modifiers.get(i, None)
#                    if not rough_mod:
#                         None
#                    else:
#                        ob.select_set(True)
#                        bpy.context.view_layer.objects.active = ob
#                        ob.modifiers["disp"].strength = mytool.my_streng
#                        ob.modifiers["subd"].levels = mytool.my_subd
      
    
        return {'FINISHED'}          


class pickerButton(Operator):
    bl_idname = "wm.picker"
    bl_label = "Pick Settings"
    bl_description="Settings Picker. Get settings from active object"
    def execute(self, context):
        mytool = bpy.context.scene.toolz
        o=bpy.context.view_layer.objects.active
        frame=bpy.context.scene.frame_current
        for m in o.modifiers:
            if m.name=="subd":
                mytool.my_subd=m.levels
                mytool.my_subd2=m.render_levels
            if m.name=="disp":
                mytool.my_streng=m.strength
                nam=o.modifiers["disp"].texture.name
                mytool.my_scal=bpy.data.textures[nam].noise_scale 

        bpy.context.scene.frame_set(frame+1)
        bpy.context.scene.frame_set(frame)

        return {'FINISHED'}  






class selectsameButton(Operator):
    bl_idname = "wm.selectsame"
    bl_label = "Select Same"
    bl_description="Select ojects with same settings"
    def execute(self, context):
        mytool = bpy.context.scene.toolz
        frame=bpy.context.scene.frame_current
        o=bpy.context.view_layer.objects.active
        for m in o.modifiers:
            if m.name=="subd":
                C=m.levels
                D=m.render_levels
            elif m.name=="disp":
                B=m.strength
                nam=o.modifiers["disp"].texture.name
                A=bpy.data.textures[nam].noise_scale 
                list=[]            
                for ob in bpy.data.objects:
                    for m in ob.modifiers:
                        if m.name=="subd":
                            if m.levels==C:
                                if m.render_levels==D:  
                                    list.append(ob)         
                        if m.name=="disp":             
                            if ob in list and m.strength==B:
                                nam=ob.modifiers["disp"].texture.name
                                if bpy.data.textures[nam].noise_scale==A:                  
                                    ob.select_set(True)
            else:
                pass 

        bpy.context.scene.frame_set(frame+1)
        bpy.context.scene.frame_set(frame)

        return {'FINISHED'}  

class joinerButton(Operator):
    bl_idname = "wm.joiner"
    bl_label = "Join Fragments"
    bl_description="If you want to join fragments back together. It will set collision to Mesh too"
    def execute(self, context):
        sel= bpy.context.selected_objects
        for ob in sel:
            bpy.context.view_layer.objects.active = ob
        bpy.ops.object.join()
        o=bpy.context.view_layer.objects.active    
        try:
            o.rigid_body.type!=None
            bpy.context.object.rigid_body.collision_shape = 'MESH'
            bpy.context.object.rigid_body.collision_margin = 0.0001
            bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')
        except:
            pass
            #print("pas de Rigid body")            
    
        return {'FINISHED'}  





class BREAK2Button(Operator):
    bl_idname = "wm.breaker2"
    bl_label = "B R E A K !"
    bl_description="For pre fractured object. It will break the object in sub object at current frame"
    def execute(self, context):
        
          
        ######################################################
        scene = bpy.context.scene
        mytool = scene.toolz        
        if mytool.my_boolTimeline == False:
            start_frame=scene.frame_start
            end_frame=scene.frame_end
        else: 
            start_frame=mytool.my_start
            end_frame=mytool.my_end       
        break_frame=scene.frame_current        
        friction= mytool.my_friction
                
        ### create collection and make it active ################################
        if mytool.my_boolCollection == True:
            colname = mytool.my_name
            colname2 = colname+"_hide"
            mastercollection = bpy.context.scene.collection  
            res=0
            res2=0
            for collec in bpy.data.collections:
                if collec.name == colname:
                    col = bpy.data.collections.get(colname)
                    res+=1
                if collec.name == colname2:
                    col = bpy.data.collections.get(colname2)
                    res2+=1                
            if res<1:    
                mycollection = bpy.data.collections.new(name=colname)
                bpy.context.scene.collection.children.link(mycollection) 
            if res2<1:    
                mycollection = bpy.data.collections.new(name=colname2)
                bpy.context.scene.collection.children.link(mycollection)                             
            layer_collection = bpy.context.view_layer.layer_collection
            for layer in layer_collection.children:
                if layer.name == colname:
                    bpy.context.view_layer.active_layer_collection = layer
            bpy.data.collections[colname2].hide_render = True
            bpy.data.collections[colname2].hide_viewport = True
            
        ob = bpy.context.active_object
        #bake action
        bpy.ops.nla.bake(frame_start=start_frame, frame_end=end_frame, visual_keying=True, clear_parents=True, bake_types={'OBJECT'})        
        scene.frame_current = break_frame
    
        #empty creation
        objects = bpy.data.objects
        EMPT = objects.get('EMPT'+ob.name)
        if EMPT is None:
            bpy.ops.object.empty_add(type='PLAIN_AXES')
            EMPT= bpy.context.active_object
            EMPT.name = 'EMPT_'+ob.name
        EMPT.show_in_front=True  

        # copy ob action to EMPT
        bpy.ops.object.select_all(action='DESELECT')
        ob.select_set(True)
        bpy.context.view_layer.objects.active = ob
        context = bpy.context
        areatype = context.area.type
        bpy.context.area.type = 'DOPESHEET_EDITOR'
        bpy.context.space_data.mode = 'ACTION'
        EMPT.animation_data_create() # !!!!!!!!!!!!!!!!! coooool
        EMPT.animation_data.action = ob.animation_data.action
        bpy.ops.action.unlink()
        bpy.context.area.type = areatype

        # separate by loose part
        fraglist=[]
        bpy.ops.object.mode_set(mode='EDIT')
        bpy.ops.mesh.select_all(action="SELECT") 
        bpy.ops.mesh.separate(type='LOOSE')
        bpy.ops.object.mode_set(mode='OBJECT') 
        bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')
        active_ob = bpy.context.view_layer.objects.active
        for i in bpy.context.selected_objects:
            if mytool.my_boolTexspc == True:
                i.data.use_auto_texspace = False
                i.data.texspace_location[0] = -i.matrix_world.translation[0]+active_ob.location[0]
                i.data.texspace_location[1] = -i.matrix_world.translation[1]+active_ob.location[1]
                i.data.texspace_location[2] = -i.matrix_world.translation[2]+active_ob.location[2]
                i.data.texspace_size[0] = 1
                i.data.texspace_size[1] = 1
                i.data.texspace_size[2] = 1
            
            fraglist.append(i)

        # rigid bodies
        for obj in fraglist:
            bpy.context.view_layer.objects.active = obj
            bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY', center='MEDIAN')
            bpy.ops.rigidbody.objects_add(type='ACTIVE') 
            obj.rigid_body.collision_shape = 'CONVEX_HULL'
            obj.rigid_body.kinematic = True
            obj.rigid_body.friction = friction
          

        #parent ob to EMPT
        bpy.context.view_layer.update()  
        EMPT.select_set(True)
        bpy.context.view_layer.objects.active = EMPT
        bpy.ops.object.parent_set()

        #add kinematic key on/off
        EMPT.select_set(False)
        end = break_frame
        for i in fraglist:
            bpy.context.view_layer.objects.active=i
            scene.frame_current = end-1
            i.rigid_body.kinematic = True
            i.keyframe_insert('rigid_body.kinematic')
            scene.frame_current = end
            i.rigid_body.kinematic = False
            i.keyframe_insert('rigid_body.kinematic') 
         
        #collection 
        if mytool.my_boolCollection == True:
            col = bpy.data.collections.get(colname)
            EMPT.select_set(True )
            sel = bpy.context.selected_objects
            
            for o in sel :
                #bpy.context.view_layer.objects.active=o
                origin_collname= o.users_collection[0].name 
                #print(o.name, origin_collname)
                origin_coll = bpy.data.collections.get(origin_collname)
                if origin_coll != None:
                    origin_coll.objects.unlink(o)
                    col.objects.link(o)   
                else:
                    mastercollection.objects.unlink(o)
                    col.objects.link(o)  
        EMPT.hide_viewport = True
        EMPT.select_set(False)
        if mytool.my_boolAutoplay == True:
            scene.frame_current = start_frame
            bpy.ops.screen.animation_play()  
            

        return {'FINISHED'}         


class KINEMATICButton(Operator):
    bl_idname = "wm.kinemat"
    bl_label = "Dynamic Keyframe"
    bl_description="Add 'Animated' keyframe on rigid body selection to trigger physics behavior"
    def execute(self, context):
        scene = bpy.context.scene
        frm =scene.frame_current
        for obj in bpy.context.selected_objects:
            obj.select_set(True)
            bpy.context.view_layer.objects.active = obj  
            bpy.ops.rigidbody.object_add()

        #print("RB ok")
        for obj in bpy.context.selected_objects:
            obj.rigid_body.kinematic = True
            obj.keyframe_insert('rigid_body.kinematic')  
        #print("on ok")      
        bpy.data.scenes["Scene"].frame_set(frm+1)
        for obj in bpy.context.selected_objects:    
            obj.rigid_body.kinematic = False
            obj.keyframe_insert('rigid_body.kinematic') 
        #print("off ok")
        
        return {'FINISHED'} 

class BAKEButton(Operator):
    bl_idname = "wm.baker"
    bl_label = "Bake to Keyframes"
    bl_description="Bake selected objects actions to keyframe. This will get ride of simulation and parenting"
    def execute(self, context):

        scene = bpy.context.scene
        mytool = scene.toolz        
        if mytool.my_boolTimeline == False:
            start_frame=scene.frame_start
            end_frame=scene.frame_end
        else:
            start_frame=mytool.my_start
            end_frame=mytool.my_end              
        bpy.ops.nla.bake(frame_start=start_frame, frame_end=end_frame, visual_keying=True, clear_parents=True, bake_types={'OBJECT'})
        sel = bpy.context.selected_objects        
        for i in sel:
            #i.select_set(True)
            bpy.context.view_layer.objects.active = i
            bpy.ops.rigidbody.object_add() # remove RB (add one before if not any to avoid error)
            bpy.ops.rigidbody.object_remove()
        return {'FINISHED'}         
 
    
class texspButton(Operator):
    bl_idname = "wm.texsp"
    bl_label = "Unify Texture Space"
    bl_description="Make same texture space for selected objects so you can use Generated UV mapping "
    def execute(self, context):

        sel= bpy.context.selected_objects
        size=1
        active_ob = bpy.context.view_layer.objects.active
        for i in sel:
            if i.type == 'MESH':
                i.show_texture_space = False
                i.data.use_auto_texspace = False
                i.data.texspace_location[0] = -i.matrix_world.translation[0]+active_ob.location[0]
                i.data.texspace_location[1] = -i.matrix_world.translation[1]+active_ob.location[1]
                i.data.texspace_location[2] = -i.matrix_world.translation[2]+active_ob.location[2]
                
                i.data.texspace_size[0] = size
                i.data.texspace_size[1] = size
                i.data.texspace_size[2] = size
        return {'FINISHED'}  
    
class rsttexspButton(Operator):
    bl_idname = "wm.rsttexsp"
    bl_label = "Reset Texture Space"
    bl_description="set texture space its default settings for selected objects"
    def execute(self, context):

        sel= bpy.context.selected_objects
        size=1
        for i in sel:
            if i.type == 'MESH':
                i.data.use_auto_texspace = True
        return {'FINISHED'}  
    
class ARBButton(Operator):
    bl_idname = "wm.arb"
    bl_label = "Active Rigid Body"
    bl_description="Add Selection As Active Rigid Body"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):
        sel = bpy.context.selected_objects 
        for ob in sel:
            if ob.type == 'MESH':
                bpy.context.view_layer.objects.active =ob 
                bpy.ops.rigidbody.objects_add(type='ACTIVE')
        return {'FINISHED'} 
class PRBButton(Operator):
    bl_idname = "wm.prb"
    bl_label = "Passive Rigid Body"
    bl_description="Add Selection As Passive Rigid Body"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):
        sel = bpy.context.selected_objects 
        for ob in sel:
            bpy.context.view_layer.objects.active =ob          
            if ob.type == 'MESH':
                bpy.ops.rigidbody.objects_add(type='PASSIVE')
        return {'FINISHED'} 
class RRBButton(Operator):
    bl_idname = "wm.rrb"
    bl_label = ""
    bl_description="Remove Selection As Rigid Body"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):
        sel = bpy.context.selected_objects 
        for ob in sel:
            bpy.context.view_layer.objects.active =ob 
            if ob.type == 'MESH':   
                bpy.ops.rigidbody.object_add()     
                bpy.ops.rigidbody.objects_remove()
        return {'FINISHED'} 

class HELPButton(Operator):
    bl_idname = "wm.help"
    bl_label = "Show Helpers"
    bl_description="Show Helpers"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):
        for o in bpy.data.objects:
            o_name=o.name
            if o_name.startswith("EMPT_"):
                o.hide_viewport=False

        return {'FINISHED'} 
class HELPButton2(Operator):
    bl_idname = "wm.help2"
    bl_label = "Hide Helpers"
    bl_description="Show Helpers"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):
        for o in bpy.data.objects:
            o_name=o.name
            if o_name.startswith("EMPT_"):
                o.hide_viewport=True

        return {'FINISHED'}     


class replaceButton(Operator):
    bl_idname = "wm.replace"
    bl_label = "Replace 2nd material"
    bl_description="For selected objects, replace 2 material by"
    def execute(self, context):
        scene = bpy.context.scene
        mytool = scene.toolz  
        mat_name=mytool.my_enum  
        new_material = bpy.data.materials.get(mat_name)
        print(mat_name)

        for ob in bpy.context.selected_objects:
            bpy.context.view_layer.objects.active=ob
            if len(ob.material_slots)>1:
                ob.data.materials[1] = new_material
        bpy.ops.wm.redraw_timer(type='DRAW_WIN_SWAP', iterations=1)

        return {'FINISHED'}  
    
    
    
# ------------------------------------------------------------------------
#    Panel in Object Mode
# ------------------------------------------------------------------------

class FracturBreakerPanel:

    bl_space_type = "VIEW_3D"   
    bl_region_type = "UI"
    bl_category = "FRACTURE"
    #bl_context = "objectmode"   
    

class MAIN_PT_PANEL(FracturBreakerPanel, bpy.types.Panel):
    bl_idname = "MAIN_PT_PANEL"
    bl_label = "FRACTURE  ITERATOR 1.7"
    bl_options = {"DEFAULT_CLOSED"}
        
#    @classmethod
#    def poll(self,context):
#        return context.object is not None    
    def draw(self, context):
        layout = self.layout
        scene = context.scene
        mytool = scene.toolz

        box = layout.box()
        split = box.split()
        col = split.column(align=True)
        row = col.row(align=True) 
        row.label(text="Cell Fracture:")
        row = col.row(align=True) 
        row.prop(mytool, "my_enu")
        row = col.row(align=True) 
        row.prop(mytool, "my_num")
        row = col.row(align=True) 
        row.prop(mytool, "my_scalx")
        row.prop(mytool, "my_scaly")
        row.prop(mytool, "my_scalz")

        if mytool.my_enu == 'P0':     
            row = col.row(align=True)  
            row.operator("wm.fracture") 
            row.scale_y = 2  
            row = col.row(align=True)  
            row.operator("wm.breaker0") 
            row.scale_y = 2.0           
            
        else:     
            row = col.row(align=True)  
            row.operator("wm.breaker")
            row.scale_y = 2.0
        col.separator()
        row = col.row(align=True) 
        row.operator("wm.arb") 
        row.operator("wm.prb") 
        row.operator("wm.rrb",icon="TRASH") 


        
        
    
    
        box = layout.box()
        split = box.split()
        col = split.column(align=True)
        row = col.row(align=True) 
        row.label(text="Loose Parts:")
        row = col.row(align=True) 
        row.operator("wm.joiner")
        row = col.row(align=True) 
        row.operator("wm.breaker2")


        box = layout.box()
        split = box.split()
        col = split.column(align=True)
        row = col.row(align=True) 
        row.label(text="Rigid Body Activation:")
        row = col.row(align=True) 
        row.operator("wm.kinemat")
        
        
        #material chooser
        box = layout.box()
        split = box.split()
        col = split.column(align=True)
        row = col.row(align=True)     
        row.prop(mytool, "my_boolMat",text="Custom Material")
        row = col.row(align=True)  
        row.label(text="Cut Material:",icon='MATERIAL')
        if mytool.my_boolMat == True:  
            
            row.prop(mytool, "my_enum")
            row = col.row(align=True) 
            row.operator("wm.replace")
        else:
            row.label(text=" Break_mat")
            

        
#        mat_name="Break_mat"
#        ng = bpy.data.materials
#        if mat_name not in ng:
#            row = col.row(align=True)  
#            row.operator("wm.mat")

class ROUGH_PT_PANEL(FracturBreakerPanel, bpy.types.Panel):
    bl_parent_id = "MAIN_PT_PANEL"
    bl_label = "Rough Cuts"
    bl_options = {"DEFAULT_CLOSED"}
        
    def draw(self, context):
        layout = self.layout
        scene = context.scene
        mytool = scene.toolz
        box = layout.box()
        split = box.split()
        col = split.column(align=True)
#        row = col.row(align=True) 
#        row.label(text="Rough Cuts:",icon="MOD_NOISE")
        row = col.row(align=True) 
        row.operator("wm.rough",icon="ADD")
        row.scale_y = 1.6
        
        row = col.row(align=True) 
        row.prop(mytool, "my_scal")
        row.prop(mytool, "my_streng")
        row = col.row(align=True) 
        row.prop(mytool, "my_subd")
        row.prop(mytool, "my_subd2")

        #row.scale_x=.1
        row = col.row(align=True)         
        row.operator("wm.selectsame",icon="RESTRICT_SELECT_OFF") 
        row.operator("wm.picker",icon="EYEDROPPER") 
        row.scale_y = 1.4
        row = col.row(align=True) 
        row.operator("wm.updrough",icon="LOOP_FORWARDS")
        row.scale_y = 2        
        col.separator()
        row = col.row(align=True) 
        row.operator("wm.unrough",icon="REMOVE") 
        row.operator("wm.resetrough",icon="TRASH") 
        
                    
class OPTION_PT_PANEL(FracturBreakerPanel, bpy.types.Panel):
    bl_parent_id = "MAIN_PT_PANEL"
    bl_label = "options"
    bl_options = {"DEFAULT_CLOSED"}
        
    def draw(self, context):
        layout = self.layout
        scene = context.scene
        mytool = scene.toolz
        
        row = layout.row(align=True)
        row.prop(mytool, "my_boolTexspc")             
        row = layout.row(align=True)
        row.operator("wm.texsp")      
        row = layout.row(align=True)
        row.operator("wm.rsttexsp")
        
        

                
        layout.separator() 
        row = layout.row(align=True)
        row.prop(mytool, "my_boolCollection")

        row.prop(mytool, "my_name", text="")
        
        row = layout.row(align=True)
        row.prop(mytool, "my_friction")
        
        row = layout.row(align=True)
        row.prop(mytool, "my_boolAutoplay")
        row = layout.row(align=True)
        row.operator("wm.help")
        row.operator("wm.help2")
        layout.separator() 
        row = layout.row(align=True)
        row.operator("wm.baker")
        row = layout.row(align=True)
        row.prop(mytool, "my_boolTimeline") 

        row = layout.row(align=True)
        row.prop(mytool, "my_start")
        row.prop(mytool, "my_end")

        

# ------------------------------------------------------------------------
#    Registration
# ------------------------------------------------------------------------

classes = (

    MyPropertiez,
    FRACTUREButton,
    BREAKButton,
    BREAK0Button,
    joinerButton,
    BREAK2Button,
    KINEMATICButton,
    BAKEButton,
    texspButton,
    rsttexspButton,
    roughButton,
    unroughButton,
    updroughButton,
    resetroughButton,
    pickerButton,
    PRBButton,
    ARBButton,
    RRBButton,
    HELPButton,
    HELPButton2,
    selectsameButton,
    MAIN_PT_PANEL,
    ROUGH_PT_PANEL, 
    OPTION_PT_PANEL, 
    replaceButton,


)



    
    
    
    
def register():

    from bpy.utils import register_class
    for cls in classes:
        register_class(cls)

    bpy.types.Scene.toolz = PointerProperty(type=MyPropertiez)
    translate.register()
    

def unregister():

    from bpy.utils import unregister_class
    for cls in reversed(classes):
        unregister_class(cls)
    del bpy.types.Scene.toolz
    translate.unregister()


if __name__ == "__main__":
    register()